//-----------------------------------------------------------------------------
// File:     HelloMath3.cpp
// Class:    None
// Parent:   None
// Children: None
// Purpose:  Tests out various mathematical functions
//-----------------------------------------------------------------------------
// The following are standard C/C++ header files.
// If a filename is enclosed inside " "  it means the header file is in the current directory.
#include <ctype.h>      // Character Types
#include <math.h>       // Mathematical Constants
#include <stdarg.h>     // Variable Argument Lists
#include <stdio.h>      // Standard Input/Output Functions
#include <stdlib.h>     // Utility Functions
#include <string.h>     // String Operations
#include <signal.h>     // Signals (Contol-C + Unix System Calls)
#include <setjmp.h>     // Nonlocal Goto (For Control-C)
#include <time.h>       // Time and Date information
#include <assert.h>     // Verify Program Assertion
#include <errno.h>      // Error Codes (Used in Unix system())
#include <float.h>      // Floating Point Constants
#include <limits.h>     // Implementation Constants
#include <stddef.h>     // Standard Definitions
#include <exception>    // Exception handling (e.g., try, catch throw)
#include <iostream>
#include <string>
#include <cstdio>
#include <cstdlib>
#include <fstream>
using namespace std;
//-----------------------------------------------------------------------------
#define PI 3.1415926535
//-----------------------------------------------------------------------------
// Prototypes for local functions (functions not called by code in other files)
//-----------------------------------------------------------------------------
bool   WriteDoubleToFileWithExponentInSpecifiedPrecision( double x, int precision, FILE *fptr );
bool  GetStringFromFile( char inputString[], unsigned long sizeOfString, FILE* fptr);
bool  GetStringFromKeyboard( char inputString[], unsigned long sizeOfString);
bool  WriteStringToScreen ( const char outputString[]);
bool  WriteStringToFile( const char outputString[], FILE *fptr);
const char* ConvertStringToLong( const char *s, long &returnValue, long defaultValue);
double ConvertFromRadiansToDegrees( long angleInRadians);
double ConvertFromDegreesToRadians( long angleInDegrees);
FILE* FileOpenWithMessageIfCannotOpen( const char *filename, const char *attribute);FILE* FileOpenWithMessageIfCannotOpen( const char *filename, const char *attribute);
bool GetDoubleRowElementsFromFile(double *array, unsigned int numberOfCols, FILE *fptr);
bool GetDoubleMatrixElementsFromFile(double *array, unsigned int numberOfRows, unsigned int numberOfCols, FILE *fptr);
bool WriteDoubleRowElemementsToFile(const double *array, unsigned int numberOfCols, int precision, FILE *fptr);
bool WriteDoubleMatrixELementsToFile(const double *array, unsigned int numberOfRows, unsigned int numberOfCols, int precision, FILE *fptr);
void AddMatrics(const double *arrayA, const double *arrayB, double *arraySum, unsigned int numberOfElements);
//-----------------------------------------------------------------------------
int  main( int numberOfCommandLineArguments, char *arrayOfCommandLineArguments[] )
{  	
	const char fileAttributeR[5]="rb";
    const char fileAttributeW[5]="w";
	FILE *pFile = FileOpenWithMessageIfCannotOpen("HelloMath3In.txt",fileAttributeR);
	if (!pFile) return 1;
	double mArray[35];
	double *Parray=mArray;
	int numberOfRows=5, numberOfCols=7;
	GetDoubleMatrixElementsFromFile(Parray,numberOfRows,numberOfCols,pFile);
	double arraySum[35];
	double *ParraySum=arraySum;
	AddMatrics(mArray, mArray, Parray, 35);
	FILE *pFileout = FileOpenWithMessageIfCannotOpen("HelloMath3Out.txt",fileAttributeW);
    WriteDoubleMatrixELementsToFile(Parray, 5, 7, 8, pFileout); 
	system("PAUSE");
	return 0;
}

//------------------------------------------------------------------------------------------------
bool WriteDoubleMatrixELementsToFile(const double *array, unsigned int numberOfRows, unsigned int numberOfCols, int precision, FILE *fptr)
{
	for (unsigned int i=0;i<numberOfRows;i++)
	{
	WriteDoubleRowElemementsToFile(array,numberOfCols,precision, fptr);
	WriteStringToFile("\n",fptr);
	}
fclose(fptr);
return 0;
}
//-----------------------------------------------------------------------------------------------------------
bool WriteDoubleRowElemementsToFile(const double *array, unsigned int numberOfCols, int precision, FILE *fptr)
{
    static int columnCounter=0;
	int addToArray4Write=(numberOfCols*columnCounter);
	int fieldwidth=precision+8;
	char format[20];
    for(unsigned int j=0;j<numberOfCols;j++)    
	{
	 WriteDoubleToFileWithExponentInSpecifiedPrecision(array[j+addToArray4Write], precision, fptr);
	}
	columnCounter=columnCounter+1;
		return 0;
}
//------------------------------------------------------------------------------------------------------
bool GetDoubleMatrixElementsFromFile(double *array, unsigned int numberOfRows, unsigned int numberOfCols, FILE *fptr)
{
	for(unsigned int i=0; i<numberOfRows; i++) 
	{
		GetDoubleRowElementsFromFile(array,numberOfCols,fptr);
	}
	return 0;
	
}
//------------------------------------------------------------------------------------------------------
bool GetDoubleRowElementsFromFile(double *array, unsigned int numberOfCols, FILE *fptr)
{
	static int counterToGetToNextRow=0;
	int addToArray=(numberOfCols*counterToGetToNextRow);
	for(unsigned int j=0; j<numberOfCols; j++)
	{
		fscanf(fptr,"%lg",array+j+addToArray);
	}
    counterToGetToNextRow=counterToGetToNextRow+1;
	return 0;
}
//__________________________________________________________________________________

void AddMatrics(const double *arrayA, const double *arrayB, double *arraySum, unsigned int numberOfElements)
{
	for(unsigned int i=0;i<numberOfElements;i++)
		{
			arraySum[i]=(arrayA[i]+arrayB[i]);
	    }
}
//____________________________________________________________________________________________

bool GetStringFromKeyboard( char inputString[], unsigned long sizeOfString)
{  
	GetStringFromFile(inputString,256,stdin);
	 return 0;
}   
//_________________________________________________________________________________
bool GetStringFromFile(char inputString[], unsigned long sizeOfString, FILE* fptr)
{
	fgets(inputString,sizeOfString,fptr);
	return 0;
}

//_____________________________________________________________________________________
const char* ConvertStringToLong( const char *s, long &returnValue, long defaultValue)
{
	returnValue = defaultValue;
	if (s)
	{
	char *pointerToCharacterAfterNumber = NULL;
	long x =strtol(s,&pointerToCharacterAfterNumber,0);
	char characterAfterNumber = pointerToCharacterAfterNumber ? *pointerToCharacterAfterNumber : 'z';
		if ( characterAfterNumber == '\0' || isspace (characterAfterNumber) )
		{
			returnValue = x;
			return pointerToCharacterAfterNumber;
        }
    }
return NULL;
}

//___________________________________________________________________________________________
bool  WriteStringToScreen ( const char outputString[])
{
	WriteStringToFile(outputString, stdout);
	return 0;
}
//___________________________________________________________________________________
bool  WriteStringToFile( const char outputString[], FILE *fptr)
{
	fprintf(fptr,outputString);
	return 0;
}
//___________________________________________________________________________________________
bool WriteDoubleToFileWithExponentInSpecifiedPrecision(double x, int precision, FILE *fptr)
{
	//fptr
	if (precision < 0 || precision >17) precision =5;
	int fieldWidth =precision +8;
	char format[20];
	sprintf(format," %%- %d.%dE", fieldWidth, precision);
    return fprintf( fptr, format, x) >0;
}
//---------------------------------------------------------------------
FILE* FileOpenWithMessageIfCannotOpen( const char *filename, const char *attribute)
{
	FILE *p2File;
	p2File=fopen(filename,attribute);//"r"
	if (p2File==NULL)
	{
		WriteStringToScreen("Sorry, could not open file that was requested, please check name and re-enter\n");
		return NULL;
	}
	return p2File;
}
//__________________________________________________________________________________________________________
